#pragma once

#include "proto/rtransfer_messages.pb.h"
#include "uuid.hpp"

#include <monitoring/monitoring.h>
#include <wangle/service/ClientDispatcher.h>

#include <unordered_map>

namespace rtransfer {
using MsgPtr = std::unique_ptr<proto::LinkMessage>;

class ClientDispatcher
    : public wangle::ClientDispatcherBase<
          wangle::Pipeline<folly::IOBufQueue &, MsgPtr>, MsgPtr> {
public:
    void read(Context *, MsgPtr in) override
    {
        auto it = promises_.find(in->req_id());
        if (it == promises_.end())
            return;

        ONE_METRIC_COUNTER_DEC("requests.remote.outgoing.concurrent");
        it->second.setValue(std::move(in));
        promises_.erase(it);
    }

    folly::Future<MsgPtr> operator()(MsgPtr arg) override
    {
        DCHECK(this->pipeline_);
        ONE_METRIC_COUNTER_INC("requests.remote.outgoing.concurrent");

        auto reqId = genUUID();
        arg->set_req_id(reqId.data(), reqId.size());
        auto result =
            promises_.emplace(std::move(reqId), folly::Promise<MsgPtr>{});
        auto f = result.first->second.getFuture();

        auto transport = this->pipeline_->getTransport();
        DCHECK(transport);
        auto base = transport->getEventBase();
        DCHECK(base);

        base->runInEventBaseThread(
            [pipeline = this->pipeline_, arg = std::move(arg)]() mutable {
                pipeline->write(std::move(arg));
            });

        return f;
    }

private:
    std::unordered_map<folly::fbstring, folly::Promise<MsgPtr>> promises_;
};

}  // namespace rtransfer
