#pragma once

#include "common.hpp"
#include "handleCache.hpp"
#include "storage.hpp"

#include <folly/FBString.h>
#include <folly/SpinLock.h>
#include <folly/String.h>
#include <folly/concurrency/ConcurrentHashMap.h>
#include <folly/futures/Future.h>
#include <monitoring/monitoring.h>
#include <wangle/channel/Handler.h>

#include <cstdint>
#include <unordered_map>

namespace rtransfer {

class FetchManager
    : public wangle::InboundHandler<std::unique_ptr<folly::IOBuf>,
          folly::Unit> {
public:
    FetchManager(StoragesMap &storages, HandleCache &handleCache);

    folly::Future<std::size_t> newFetch(std::uint64_t requestId,
        folly::fbstring destStorageId, folly::fbstring destFileId,
        folly::fbstring fileGuid, std::uint64_t offset, std::size_t size,
        folly::Function<void(std::uint64_t, std::size_t)> notifyCb);

    void read(Context *ctx, std::unique_ptr<folly::IOBuf> buf) override;

    void cancelFetch(std::uint64_t requestId);

    void setTotalSize(std::uint64_t reqId, std::size_t size);

private:
    struct FetchInfo {
        mutable folly::SpinLock lock;
        folly::fbstring storageId;
        folly::fbstring fileId;
        folly::fbstring fileGuid;
        std::uint64_t offset;
        std::size_t size;
        std::size_t wrote = 0;
        folly::Promise<std::size_t> promise;
        folly::Function<void(std::uint64_t, std::size_t)> notifyCb;
    };

    using Map =
        folly::ConcurrentHashMap<std::uint64_t, std::shared_ptr<FetchInfo>>;

    void doRead(std::unique_ptr<folly::IOBuf> buf);

    void cancelFetch(Map::const_iterator it, folly::StringPiece reason);

    void maybeFulfillPromise(Map::const_iterator it);

    StoragesMap &storages_;
    HandleCache &handleCache_;
    Map fetchInfo_;
};

}  // namespace rtransfer
